/*============================================================================
  File:     HackAttachLogBackup.sql

  Summary:  Hack-attach log tail backup demo.

  Date:     June 2010

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  (c) 2011, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

IF DATABASEPROPERTYEX ('DBMaint2008', 'Version') > 0
	DROP DATABASE DBMaint2008;

-- Imagine a complete server meltdown
-- We've just got a log file and some backups
-- Now what?

-- Restoring just the backups, we get:
RESTORE DATABASE DBMaint2008
	FROM DISK = 'D:\SQLskills\DemoBackups\DBMaint_Full_HAD.bck'
WITH REPLACE, NORECOVERY,
MOVE 'DBMaint2008' TO 'C:\Program Files\Microsoft SQL Server\MSSQL10.SQLDEV01\MSSQL\DATA\DBMaint2008.mdf',
MOVE 'DBMaint2008_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL10.SQLDEV01\MSSQL\DATA\DBMaint2008.ldf';
GO

-- And all other backups...
RESTORE LOG DBMaint2008
	FROM DISK = 'D:\SQLskills\DemoBackups\DBMaint_Log1_HAD.bck'
WITH NORECOVERY;
GO

-- Bring it online
RESTORE DATABASE DBMaint2008 WITH RECOVERY;
GO

-- Check the data
SELECT * FROM DBMaint2008.dbo.TestTable;
GO

-- What about getting the data from the log file?

-- Get rid of the newly restored database
DROP DATABASE DBMaint2008;
GO

-- Create a dummy database
CREATE DATABASE DBMaint2008;
GO

-- Shut the database down
ALTER DATABASE DBMaint2008 SET OFFLINE;
GO

-- Delete the data file and swap in the
-- old log file

-- Restart the database
ALTER DATABASE DBMaint2008 SET ONLINE;
GO

-- Take a log backup?
BACKUP LOG DBMaint2008 TO
	DISK = 'D:\SQLskills\DBMaint2008_tail.bck'
WITH INIT;
GO

-- Use the special syntax!
BACKUP LOG DBMaint2008 TO
	DISK = 'D:\SQLskills\DBMaint2008_tail.bck'
WITH INIT, NO_TRUNCATE;
GO

-- Now we can restore everything
-- First the full backup...
RESTORE DATABASE DBMaint2008
	FROM DISK = 'D:\SQLskills\DemoBackups\DBMaint_Full_HAD.bck'
WITH REPLACE, NORECOVERY,
MOVE 'DBMaint2008' TO 'C:\Program Files\Microsoft SQL Server\MSSQL10.SQLDEV01\MSSQL\DATA\DBMaint2008.mdf',
MOVE 'DBMaint2008_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL10.SQLDEV01\MSSQL\DATA\DBMaint2008.ldf';
GO

-- And all other backups...
RESTORE LOG DBMaint2008
	FROM DISK = 'D:\SQLskills\DemoBackups\DBMaint_Log1_HAD.bck'
WITH NORECOVERY;
GO

-- And restore the tail of the log backup
-- we just took
RESTORE LOG DBMaint2008
	FROM DISK = 'D:\SQLskills\DBMaint2008_tail.bck'
WITH NORECOVERY;
GO

-- And finalize recovery
RESTORE DATABASE DBMaint2008 WITH RECOVERY;
GO

-- Check the data
SELECT * FROM DBMaint2008.dbo.TestTable;
GO